@@ -13,12 +13,12 @@
 #include "output_model.h"
 
 #include <kscreen/config.h>
+#include <kscreen/configmonitor.h>
 #include <kscreen/getconfigoperation.h>
 #include <kscreen/log.h>
 #include <kscreen/output.h>
 #include <kscreen/setconfigoperation.h>
 
-#include <KAboutData>
 #include <KConfigGroup>
 #include <KLocalizedString>
 #include <KPluginFactory>
@@ -30,22 +30,14 @@ K_PLUGIN_FACTORY_WITH_JSON(KCMDisplayConfigurationFactory, "kcm_kscreen.json", r
 
 using namespace KScreen;
 
-KCMKScreen::KCMKScreen(QObject *parent, const QVariantList &args)
-    : KQuickAddons::ConfigModule(parent, args)
+KCMKScreen::KCMKScreen(QObject *parent, const KPluginMetaData &data, const QVariantList &args)
+    : KQuickAddons::ConfigModule(parent, data, args)
 {
     qmlRegisterType<OutputModel>();
     qmlRegisterType<KScreen::Output>("org.kde.private.kcm.kscreen", 1, 0, "Output");
     qmlRegisterUncreatableType<Control>("org.kde.private.kcm.kscreen", 1, 0, "Control", QStringLiteral("Provides only the OutputRetention enum class"));
     Log::instance();
 
-    KAboutData *about = new KAboutData(QStringLiteral("kcm_kscreen"),
-                                       i18n("Display Configuration"),
-                                       QStringLiteral(KSCREEN_VERSION),
-                                       i18n("Manage and configure monitors and displays"),
-                                       KAboutLicense::GPL,
-                                       i18n("Copyright © 2019 Roman Gilg"));
-    about->addAuthor(i18n("Roman Gilg"), i18n("Developer"), QStringLiteral("subdiff@gmail.com"));
-    setAboutData(about);
     setButtons(Apply);
 
     m_loadCompressor = new QTimer(this);
@@ -55,13 +47,15 @@ KCMKScreen::KCMKScreen(QObject *parent, const QVariantList &args)
 
     m_orientationSensor = new OrientationSensor(this);
     connect(m_orientationSensor, &OrientationSensor::availableChanged, this, &KCMKScreen::orientationSensorAvailableChanged);
+
+    connect(KScreen::ConfigMonitor::instance(), &KScreen::ConfigMonitor::configurationChanged, this, &KCMKScreen::updateFromBackend);
 }
 
 void KCMKScreen::configReady(ConfigOperation *op)
 {
     qCDebug(KSCREEN_KCM) << "Reading in config now.";
     if (op->hasError()) {
-        m_config.reset();
+        m_configHandler.reset();
         Q_EMIT backendError();
         return;
     }
@@ -70,7 +64,7 @@ void KCMKScreen::configReady(ConfigOperation *op)
     const bool autoRotationSupported = config->supportedFeatures() & (KScreen::Config::Feature::AutoRotation | KScreen::Config::Feature::TabletMode);
     m_orientationSensor->setEnabled(autoRotationSupported);
 
-    m_config->setConfig(config);
+    m_configHandler->setConfig(config);
     setBackendReady(true);
     Q_EMIT perOutputScalingChanged();
     Q_EMIT primaryOutputSupportedChanged();
@@ -91,30 +85,46 @@ void KCMKScreen::save()
 
 void KCMKScreen::revertSettings()
 {
-    if (!m_config) {
+    if (!m_configHandler) {
         setNeedsSave(false);
         return;
     }
     if (!m_settingsReverted) {
-        m_config->revertConfig();
+        m_configHandler->revertConfig();
         m_settingsReverted = true;
         doSave(true);
         load(); // reload the configuration
         Q_EMIT settingsReverted();
+        m_stopUpdatesFromBackend = false;
     }
 }
 
+void KCMKScreen::setStopUpdatesFromBackend(bool value)
+{
+    m_stopUpdatesFromBackend = value;
+}
+
+void KCMKScreen::updateFromBackend()
+{
+    if (needsSave() || m_stopUpdatesFromBackend) {
+        return;
+    }
+
+    m_loadCompressor->start();
+}
+
 void KCMKScreen::doSave(bool force)
 {
-    if (!m_config) {
+    if (!m_configHandler) {
         Q_EMIT errorOnSave();
         return;
     }
 
-    auto config = m_config->config();
+    auto config = m_configHandler->config();
 
     bool atLeastOneEnabledOutput = false;
-    for (const KScreen::OutputPtr &output : config->outputs()) {
+    const auto outputs = config->outputs();
+    for (const KScreen::OutputPtr &output : outputs) {
         KScreen::ModePtr mode = output->currentMode();
 
         atLeastOneEnabledOutput |= output->isEnabled();
@@ -133,42 +143,52 @@ void KCMKScreen::doSave(bool force)
 
     if (!atLeastOneEnabledOutput && !force) {
         Q_EMIT dangerousSave();
-        m_config->checkNeedsSave();
+        m_configHandler->checkNeedsSave();
         return;
     }
 
     if (!Config::canBeApplied(config)) {
         Q_EMIT errorOnSave();
-        m_config->checkNeedsSave();
+        m_configHandler->checkNeedsSave();
         return;
     }
 
     if (!perOutputScaling()) {
         writeGlobalScale();
     }
-    m_config->writeControl();
+    m_configHandler->writeControl();
 
     // Store the current config, apply settings. Block until operation is
     // completed, otherwise ConfigModule might terminate before we get to
     // execute the Operation.
     auto *op = new SetConfigOperation(config);
+    m_stopUpdatesFromBackend = true;
     op->exec();
 
-    // The 1000ms is a legacy value tested to work for randr having
-    // enough time to change configuration.
-    QTimer::singleShot(1000, this, [this]() {
-        if (!m_config) {
+    const auto updateInitialData = [this]() {
+        if (!m_configHandler) {
             setNeedsSave(false);
             return;
         }
-        m_config->updateInitialData();
+        m_configHandler->updateInitialData();
 
-        if (!m_settingsReverted && m_config->shouldTestNewSettings()) {
-            Q_EMIT showRevertWarning();
+        if (!m_settingsReverted && m_configHandler->shouldTestNewSettings()) {
+            QTimer::singleShot(100, this, [this]() {
+                Q_EMIT showRevertWarning();
+            });
         } else {
             m_settingsReverted = false;
+            m_stopUpdatesFromBackend = false;
         }
-    });
+    };
+
+    if (m_configHandler->config()->supportedFeatures() & (KScreen::Config::Feature::SynchronousOutputChanges)) {
+        updateInitialData();
+    } else {
+        // The 1000ms is a legacy value tested to work for randr having
+        // enough time to change configuration.
+        QTimer::singleShot(1000, this, updateInitialData);
+    }
 }
 
 bool KCMKScreen::backendReady() const
@@ -187,18 +207,18 @@ void KCMKScreen::setBackendReady(bool ready)
 
 OutputModel *KCMKScreen::outputModel() const
 {
-    if (!m_config) {
+    if (!m_configHandler) {
         return nullptr;
     }
-    return m_config->outputModel();
+    return m_configHandler->outputModel();
 }
 
 void KCMKScreen::identifyOutputs()
 {
-    if (!m_config || !m_config->initialConfig() || m_outputIdentifier) {
+    if (!m_configHandler || !m_configHandler->initialConfig() || m_outputIdentifier) {
         return;
     }
-    m_outputIdentifier.reset(new OutputIdentifier(m_config->initialConfig(), this));
+    m_outputIdentifier.reset(new OutputIdentifier(m_configHandler->initialConfig(), this));
     connect(m_outputIdentifier.get(), &OutputIdentifier::identifiersFinished, this, [this]() {
         m_outputIdentifier.reset();
     });
@@ -206,10 +226,10 @@ void KCMKScreen::identifyOutputs()
 
 QSize KCMKScreen::normalizeScreen() const
 {
-    if (!m_config) {
+    if (!m_configHandler) {
         return QSize();
     }
-    return m_config->normalizeScreen();
+    return m_configHandler->normalizeScreen();
 }
 
 bool KCMKScreen::screenNormalized() const
@@ -219,34 +239,34 @@ bool KCMKScreen::screenNormalized() const
 
 bool KCMKScreen::perOutputScaling() const
 {
-    if (!m_config || !m_config->config()) {
+    if (!m_configHandler || !m_configHandler->config()) {
         return false;
     }
-    return m_config->config()->supportedFeatures().testFlag(Config::Feature::PerOutputScaling);
+    return m_configHandler->config()->supportedFeatures().testFlag(Config::Feature::PerOutputScaling);
 }
 
 bool KCMKScreen::primaryOutputSupported() const
 {
-    if (!m_config || !m_config->config()) {
+    if (!m_configHandler || !m_configHandler->config()) {
         return false;
     }
-    return m_config->config()->supportedFeatures().testFlag(Config::Feature::PrimaryDisplay);
+    return m_configHandler->config()->supportedFeatures().testFlag(Config::Feature::PrimaryDisplay);
 }
 
 bool KCMKScreen::outputReplicationSupported() const
 {
-    if (!m_config || !m_config->config()) {
+    if (!m_configHandler || !m_configHandler->config()) {
         return false;
     }
-    return m_config->config()->supportedFeatures().testFlag(Config::Feature::OutputReplication);
+    return m_configHandler->config()->supportedFeatures().testFlag(Config::Feature::OutputReplication);
 }
 
 bool KCMKScreen::autoRotationSupported() const
 {
-    if (!m_config || !m_config->config()) {
+    if (!m_configHandler || !m_configHandler->config()) {
         return false;
     }
-    return m_config->config()->supportedFeatures() & (KScreen::Config::Feature::AutoRotation | KScreen::Config::Feature::TabletMode);
+    return m_configHandler->config()->supportedFeatures() & (KScreen::Config::Feature::AutoRotation | KScreen::Config::Feature::TabletMode);
 }
 
 bool KCMKScreen::orientationSensorAvailable() const
@@ -256,10 +276,10 @@ bool KCMKScreen::orientationSensorAvailable() const
 
 bool KCMKScreen::tabletModeAvailable() const
 {
-    if (!m_config || !m_config->config()) {
+    if (!m_configHandler || !m_configHandler->config()) {
         return false;
     }
-    return m_config->config()->tabletModeAvailable();
+    return m_configHandler->config()->tabletModeAvailable();
 }
 
 void KCMKScreen::setScreenNormalized(bool normalized)
@@ -292,30 +312,30 @@ void KCMKScreen::load()
     // signal its disappearance first before deleting and replacing it.
     // We take the m_config pointer so outputModel() will return null,
     // gracefully cleaning up the QML side and only then we will delete it.
-    auto *oldConfig = m_config.release();
+    auto *oldConfig = m_configHandler.release();
     if (oldConfig) {
         emit outputModelChanged();
         delete oldConfig;
     }
 
-    m_config.reset(new ConfigHandler(this));
+    m_configHandler.reset(new ConfigHandler(this));
     Q_EMIT perOutputScalingChanged();
-    connect(m_config.get(), &ConfigHandler::outputModelChanged, this, &KCMKScreen::outputModelChanged);
-    connect(m_config.get(), &ConfigHandler::outputConnect, this, [this](bool connected) {
+    connect(m_configHandler.get(), &ConfigHandler::outputModelChanged, this, &KCMKScreen::outputModelChanged);
+    connect(m_configHandler.get(), &ConfigHandler::outputConnect, this, [this](bool connected) {
         Q_EMIT outputConnect(connected);
         setBackendReady(false);
 
         // Reload settings delayed such that daemon can update output values.
         m_loadCompressor->start();
     });
-    connect(m_config.get(), &ConfigHandler::screenNormalizationUpdate, this, &KCMKScreen::setScreenNormalized);
-    connect(m_config.get(), &ConfigHandler::retentionChanged, this, &KCMKScreen::outputRetentionChanged);
+    connect(m_configHandler.get(), &ConfigHandler::screenNormalizationUpdate, this, &KCMKScreen::setScreenNormalized);
+    connect(m_configHandler.get(), &ConfigHandler::retentionChanged, this, &KCMKScreen::outputRetentionChanged);
 
     // This is a queued connection so that we can fire the event from
     // within the save() call in case it failed.
-    connect(m_config.get(), &ConfigHandler::needsSaveChecked, this, &KCMKScreen::continueNeedsSaveCheck, Qt::QueuedConnection);
+    connect(m_configHandler.get(), &ConfigHandler::needsSaveChecked, this, &KCMKScreen::continueNeedsSaveCheck, Qt::QueuedConnection);
 
-    connect(m_config.get(), &ConfigHandler::changed, this, &KCMKScreen::changed);
+    connect(m_configHandler.get(), &ConfigHandler::changed, this, &KCMKScreen::changed);
 
     connect(new GetConfigOperation(), &GetConfigOperation::finished, this, &KCMKScreen::configReady);
 
@@ -354,7 +374,8 @@ void KCMKScreen::writeGlobalScale()
     // Scaling the fonts makes sense if you don't also set a font DPI, but we NEED to set a font
     // DPI for both PlasmaShell which does it's own thing, and for KDE4/GTK2 applications.
     QString screenFactors;
-    for (const auto &output : m_config->config()->outputs()) {
+    const auto outputs = m_configHandler->config()->outputs();
+    for (const auto &output : outputs) {
         screenFactors.append(output->name() + QLatin1Char('=') + QString::number(m_globalScale) + QLatin1Char(';'));
     }
     config->group("KScreen").writeEntry("ScreenScaleFactors", screenFactors);
@@ -418,8 +439,8 @@ void KCMKScreen::setGlobalScale(qreal scale)
         return;
     }
     m_globalScale = scale;
-    if (m_config) {
-        m_config->checkNeedsSave();
+    if (m_configHandler) {
+        m_configHandler->checkNeedsSave();
     } else {
         continueNeedsSaveCheck(false);
     }
@@ -429,18 +450,18 @@ void KCMKScreen::setGlobalScale(qreal scale)
 
 int KCMKScreen::outputRetention() const
 {
-    if (!m_config) {
+    if (!m_configHandler) {
         return -1;
     }
-    return m_config->retention();
+    return m_configHandler->retention();
 }
 
 void KCMKScreen::setOutputRetention(int retention)
 {
-    if (!m_config) {
+    if (!m_configHandler) {
         return;
     }
-    m_config->setRetention(retention);
+    m_configHandler->setRetention(retention);
 }
 
 #include "kcm.moc"
